==> sems-0.8.12.tar.gz: sems-0.8.12/AmRtpStream.h <==
/*
 * $Id: AmRtpStream.h,v 1.13.2.5 2004/11/11 12:11:35 rco Exp $
 *
 * Copyright (C) 2002-2003 Fhg Fokus
 *
 * This file is part of sems, a free SIP media server.
 *
 * sems is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * sems is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _RtpStream_h_
#define _RtpStream_h_

#include "AmThread.h"
#include "AmSdp.h"
#include "AmAudio.h"
#include "AmRtpPacketTracer.h"

#include <netinet/in.h>

#include <string>
#include <map>
using std::string;
using std::map;

#define DEAD_RTP_TIME (5*60) /* 5 minutes */

struct amci_payload_t;
class AmRtpStreamSet;
class AmRtpScheduler;

struct RtpSync {

    bool           init;
    unsigned int   begin_ts;
    struct timeval begin_time;
    unsigned int   last_ts;
    struct timeval last_time;

    RtpSync()
	: init(false) {}

    void begin(unsigned int ts,AmRtpScheduler* sched);
    void sync(AmRtpScheduler* sched);
    unsigned int computeTimestamp(AmRtpScheduler* sched);
};

/**
 * Rtp stream high level interface.
 */
class AmRtpStream 
{
private:
    AmSharedVar<bool> runcond;
    AmMutex           runmutex;

    int         payload;
    int         last_payload;
    string      mime_type;

    string             r_host;
    unsigned short     r_port;
    struct sockaddr_in r_saddr;

    unsigned short     l_port;
    struct sockaddr_in l_saddr;
    int                l_sd;

    RtpSync        send_s;
    RtpSync        recv_s;

    struct timeval last_recv_time;

    bool           recv_offset_i;
    unsigned int   recv_offset;

    unsigned int   sequence;
    unsigned int   l_ssrc;
    unsigned int   r_ssrc;
    bool           r_ssrc_i;

    unsigned int   user_ts;
    bool           blocking;

    bool           passive; // symmetric RTP

    AmRtpScheduler* sched;
    AmThread*       thread;

    /** Initializes a new random local port, and sets own attributes properly. */
    void setLocalPort();

    /** Send FROM audio_provider */
    int send( unsigned int ts, AmAudio* audio_provider, int samples );
    /** Receive TO audio_provider */
    int receive( unsigned int ts, AmAudio* audio_provider, bool record, 
		 AmRtpPacketTracer* tracer = NULL );

    unsigned int computeTimestamp();

 public:
    /** Allocates resources for future use of RTP. */
    AmRtpStream(AmThread* _t=0);
    /** Stops the stream and frees all resources. */
    ~AmRtpStream();

    /**
     * This function must be called before setLocalPort, because
     * setLocalPort will bind the socket and it will be not
     * possible to change the IP later
     */
    void setLocalIP(const string& ip);
	    
    /** 
     * Gets RTP port number. If no RTP port in assigned, assigns a new one.
     * @return local RTP port. 
     */
    int getLocalPort();

    /** 
     * Gets remote RTP port.
     * @return remote RTP port.
     */
    int getRPort();

    void setRAddr(const string& addr, unsigned short port);

    /** Symmetric RTP: passive mode */
    void setPassiveMode(bool p) { passive = p; }

    /**
     * Enables RTP stream.
     * @param sdp_payload payload from the SDP message.
     * @warning start() must have been called so that play and record work.
     * @warning It should be called only if the stream has been completly initialized,
     * @warning and only once per session. Use resume() then.
     */
    void start(const SdpPayload& sdp_payload);

    /**
     * Stops RTP stream.
     */
    void pause();

    /**
     * Resume a paused RTP stream.
     */
    void resume();

    /** Plays from input audio provider to RTP stream. */
    void play(   AmAudio* audio_provider, int max_play_time = -1 );

    /** Records from RTP stream to output audio provider. */
    int  record( AmAudio* audio_provider, int max_rec_time, float& length );

    /** Support a duplex audio connection between audio_play and audio_rec. */
    int  duplex( AmAudio* audio_play, AmAudio* audio_rec, int max_time = -1 );

    /**
     * Play an audio file over RTP (Format: Wav).
     * @param audio_file    File name including absolute path.
     * @param max_play_time Maximum play time (ex. Prepaid).
     *                      -1 for unlimited.
     */
    void play(const string& audio_file, int max_play_time = -1);

    /**
     * Records from RTP stream to an audio file (Format: Wav, signed, 16 bit).
     * @param audio_file File name including absolute path.
     * @param playload Payload ID.
     * @param max_rec_time Maximum record time before automatic abort (in seconds).
     * @return size of recorded data.
     * @todo Use pre-defined or user prefered file format.
     */
    int record(const string& audio_file, int max_rec_time, float& length);

    friend class AmRtpStreamSet;
};

struct AmRtpStreamInfo
{
    enum StreamType { 
	ST_Receive=1, 
	ST_Send=2,
	ST_Duplex=3
    };

    StreamType   type;
    AmAudio*     audio_play;
    AmAudio*     audio_rec;
    bool         ts_offset_i;
    unsigned int ts_offset;

    AmRtpStreamInfo(StreamType type, 
		    AmAudio* audio_play = NULL, 
		    AmAudio* audio_rec = NULL);
};

class AmRtpStreamSet
{
public:
    typedef map<AmRtpStream*,AmRtpStreamInfo> StreamSet;

    AmRtpStreamSet();
    ~AmRtpStreamSet();

    void addStream(AmRtpStream* str, const AmRtpStreamInfo& info);
    void removeStream(AmRtpStream* str);
    void setStrInfo(AmRtpStream* str, const AmRtpStreamInfo& info);

    // send & receive
    int exchange();

    unsigned int getTS() { return ts_set; }

    void enableRTPTrace(const string& trace_file);

private:
    AmRtpScheduler* sched;
    RtpSync         sync;
    
    unsigned int    ts_set;
    int             nb_packet;
    StreamSet       stream_set;

    struct timeval ts_clock_begin;

    auto_ptr<AmRtpPacketTracer> tracer;
};


#endif

// Local Variables:
// mode:C++
// End:

==> sems-0.9.0.tar.gz: sems-0.9.0/AmRtpStream.h <==
/*
 * $Id: AmRtpStream.h,v 1.25 2005/01/11 15:58:01 sayer Exp $
 *
 * Copyright (C) 2002-2003 Fhg Fokus
 *
 * This file is part of sems, a free SIP media server.
 *
 * sems is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * sems is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _RtpStream_h_
#define _RtpStream_h_

#include "AmThread.h"
#include "AmSdp.h"
#include "AmAudio.h"
#include "AmRtpPacketTracer.h"

#include <netinet/in.h>

#include <string>
#include <map>
using std::string;
using std::map;

#define DEAD_RTP_TIME (5*60) /* 5 minutes */

struct amci_payload_t;
class AmRtpStreamSet;
class AmRtpScheduler;

struct RtpSync {

    bool           init;
    unsigned int   begin_ts;
    struct timeval begin_time;
    unsigned int   last_ts;
    struct timeval last_time;

    RtpSync()
	: init(false) {}

    void begin(unsigned int ts,AmRtpScheduler* sched);
    void sync(AmRtpScheduler* sched);
    void sync_cond(AmRtpScheduler* sched); // only wait for timer when we have to wait
    unsigned int computeTimestamp(AmRtpScheduler* sched);
};

/**
 * Rtp stream high level interface.
 */
class AmRtpStream 
{
private:
    AmSharedVar<bool> runcond;
    AmMutex           runmutex;

    /** 
	Internal payload (only different from 
	payload if using dynamic payloads).
    */
    int         int_payload;

    /**
       Remote payload (only different from 
       int_payload if using dynamic payloads)
     */
    int         payload;

    /**
       Payload of last received packet.
       Usefull to detect talk spurt, looking 
       for comfort noise packets.
     */
    int         last_payload;

    //string      mime_type;
    string      format_parameters;

    string             r_host;
    unsigned short     r_port;
#ifdef SUPPORT_IPV6
    struct sockaddr_storage r_saddr;
    struct sockaddr_storage l_saddr;
#else
    struct sockaddr_in r_saddr;
    struct sockaddr_in l_saddr;
#endif
    unsigned short     l_port;
    int                l_sd;

    RtpSync        send_s;
    RtpSync        recv_s;

    struct timeval last_recv_time;

    bool           recv_offset_i;
    unsigned int   recv_offset;

    unsigned int   sequence;
    unsigned int   l_ssrc;
    unsigned int   r_ssrc;
    bool           r_ssrc_i;

    unsigned int   user_ts;
    bool           blocking;

    /* symmetric RTP */
    bool              passive;      // passive mode ?
    AmSharedVar<bool> first_recved; // first packet received ?

    bool mute;

    AmRtpScheduler* sched;
    AmThread*       thread;

    /** Initializes a new random local port, and sets own attributes properly. */
    void setLocalPort();

    /** Send FROM audio_provider */
    int send( unsigned int ts, AmAudio* audio_provider, int samples );
    /** Receive TO audio_provider */
    int receive( unsigned int ts, AmAudio* audio_provider, bool record, 
		 AmRtpPacketTracer* tracer = NULL );

    unsigned int computeTimestamp();

 public:
    /** Allocates resources for future use of RTP. */
    AmRtpStream(AmThread* _t=0);
    /** Stops the stream and frees all resources. */
    ~AmRtpStream();

    /**
     * This function must be called before setLocalPort, because
     * setLocalPort will bind the socket and it will be not
     * possible to change the IP later
     */
    void setLocalIP(const string& ip);
	    
    /** 
     * Gets RTP port number. If no RTP port in assigned, assigns a new one.
     * @return local RTP port. 
     */
    int getLocalPort();

    /** 
     * Gets remote RTP port.
     * @return remote RTP port.
     */
    int getRPort();

    void setRAddr(const string& addr, unsigned short port);

    /** Symmetric RTP: passive mode ? */
    void setPassiveMode(bool p) { passive = p; }

    /**
     * Enables RTP stream.
     * @param sdp_payload payload from the SDP message.
     * @warning start() must have been called so that play and record work.
     * @warning It should be called only if the stream has been completly initialized,
     * @warning and only once per session. Use resume() then.
     */
    void start(const SdpPayload& sdp_payload);

    /**
     * Stops RTP stream.
     */
    void pause();

    /**
     * Resume a paused RTP stream.
     */
    void resume();

    /**
     * Report an ICMP error.
     */
    void icmpError();

    /** Plays from input audio provider to RTP stream. */
    void play(   AmAudio* audio_provider, int max_play_time = -1 );

    /** Records from RTP stream to output audio provider. */
    int  record( AmAudio* audio_provider, int max_rec_time );

    /** Support a duplex audio connection between audio_play and audio_rec. */
    int  duplex( AmAudio* audio_play, AmAudio* audio_rec, int max_time = -1 );

    /**
     * Play an audio file over RTP (Format: Wav).
     * @param audio_file    File name including absolute path.
     * @param max_play_time Maximum play time (ex. Prepaid).
     *                      -1 for unlimited.
     */
    void play(const string& audio_file, int max_play_time = -1);

    /**
     * Records from RTP stream to an audio file (Format: Wav, signed, 16 bit).
     * @param audio_file File name including absolute path.
     * @param playload Payload ID.
     * @param max_rec_time Maximum record time before automatic abort (in seconds).
     * @return size of recorded data.
     * @todo Use pre-defined or user prefered file format.
     */
    int record(const string& audio_file, int max_rec_time);
    
    friend class AmRtpStreamSet;
};

struct AmRtpStreamInfo
{
    enum StreamType { 
	ST_Receive=1, 
	ST_Send=2,
	ST_Duplex=3
    };

    StreamType   type;
    AmAudio*     audio_play;
    AmAudio*     audio_rec;
    bool         ts_offset_i;
    unsigned int ts_offset;

    AmRtpStreamInfo(StreamType type, 
		    AmAudio* audio_play = NULL, 
		    AmAudio* audio_rec = NULL);
};

class AmRtpStreamSet
{
public:
    typedef map<AmRtpStream*,AmRtpStreamInfo> StreamSet;

    AmRtpStreamSet();
    ~AmRtpStreamSet();

    void addStream(AmRtpStream* str, const AmRtpStreamInfo& info);
    void removeStream(AmRtpStream* str);
    void setStrInfo(AmRtpStream* str, const AmRtpStreamInfo& info);

    // send & receive
    int exchange();

    unsigned int getTS() { return ts_set; }

    void enableRTPTrace(const string& trace_file);

private:
    AmRtpScheduler* sched;
    RtpSync         sync;
    
    unsigned int    ts_set;
    int             nb_packet;
    StreamSet       stream_set;

    struct timeval ts_clock_begin;

    auto_ptr<AmRtpPacketTracer> tracer;
};


#endif

// Local Variables:
// mode:C++
// End:
